@@ -2,7 +2,26 @@ from PIL import Image
 
 import rawapi
 
+# import basic elements directly, so the caller
+# doesn't have to import rawapi if they need them.
+from rawapi import SaneCapabilities
+from rawapi import SaneConstraint
+from rawapi import SaneConstraintType
+from rawapi import SaneException
+from rawapi import SaneStatus
+from rawapi import SaneUnit
+from rawapi import SaneValueType
+
+
 __all__ = [
+    'SaneCapabilities',
+    'SaneConstraint',
+    'SaneConstraintType',
+    'SaneException',
+    'SaneStatus',
+    'SaneValueType',
+    'SaneUnit',
+
     'Scanner',
     'ScannerOption',
     'get_devices',
@@ -41,12 +60,12 @@ class ScannerOption(object):
     name = ""
     title = ""
     desc = ""
-    val_type = rawapi.SaneValueType(rawapi.SaneValueType.INT)
-    unit = rawapi.SaneUnit(rawapi.SaneUnit.NONE)
+    val_type = SaneValueType(SaneValueType.INT)
+    unit = SaneUnit(SaneUnit.NONE)
     size = 4
-    capabilities = rawapi.SaneCapabilities(rawapi.SaneCapabilities.NONE)
+    capabilities = SaneCapabilities(SaneCapabilities.NONE)
 
-    constraint_type = rawapi.SaneConstraintType(rawapi.SaneConstraintType.NONE)
+    constraint_type = SaneConstraintType(SaneConstraintType.NONE)
     constraint = None
 
     def __init__(self, scanner, idx):
@@ -56,14 +75,15 @@ class ScannerOption(object):
     @staticmethod
     def build_from_rawapi(scanner, opt_idx, opt_raw):
         opt = ScannerOption(scanner, opt_idx)
-        opt.name = opt_raw.name
-        opt.title = opt_raw.title
-        opt.desc = opt_raw.desc # TODO : multi-line
-        opt.val_type = rawapi.SaneValueType(opt_raw.type)
-        opt.unit = rawapi.SaneUnit(opt_raw.unit)
+        opt.name = opt_raw
+        opt.name = opt_raw.name.decode('utf-8')
+        opt.title = opt_raw.title.decode('utf-8')
+        opt.desc = opt_raw.desc.decode('utf-8')  # TODO : multi-line
+        opt.val_type = SaneValueType(opt_raw.type)
+        opt.unit = SaneUnit(opt_raw.unit)
         opt.size = opt_raw.size
-        opt.capabilities = rawapi.SaneCapabilities(opt_raw.cap)
-        opt.constraint_type = rawapi.SaneConstraintType(opt_raw.constraint_type)
+        opt.capabilities = SaneCapabilities(opt_raw.cap)
+        opt.constraint_type = SaneConstraintType(opt_raw.constraint_type)
         opt.constraint = opt.constraint_type.get_pyobj_constraint(
                 opt_raw.constraint)
         return opt
@@ -107,7 +127,7 @@ class ImgUtil(object):
         }[mode]
         width = pixels_per_line
         height = (len(raw) / (width * nb_colors))
-        return Image.frombuffer(mode, (width, height), raw, "raw", mode, 0, 1)
+        return Image.frombuffer(mode, (int(width), int(height)), raw, "raw", mode, 0, 1)
 
     @staticmethod
     def __raw_16_to_img(raw, mode, pixels_per_line):
@@ -174,7 +194,7 @@ class SimpleScanSession(object):
             try:
                 while True:
                     self.read()
-            except EOFError, exc:
+            except EOFError:
                 pass
         return self.__img
 
@@ -230,7 +250,7 @@ class MultiScanSession(object):
             try:
                 while True:
                     self.read()
-            except EOFError, exc:
+            except EOFError:
                 pass
         return self.__imgs[idx]
 
@@ -245,10 +265,10 @@ class MultiScanSession(object):
 class Scanner(object):
     def __init__(self, name, vendor="Unknown", model="Unknown",
                  dev_type="Unknown"):
-        self.name = name
-        self.vendor = vendor
-        self.model = model
-        self.dev_type = dev_type
+        self.name = name.decode('utf-8')
+        self.vendor = vendor.decode('utf-8')
+        self.model = model.decode('utf-8')
+        self.dev_type = dev_type.decode('utf-8')
         self.__options = None  # { "name" : ScannerOption }
 
     @staticmethod
@@ -289,7 +309,7 @@ class Scanner(object):
         self.__options = {}
         for opt_idx in range(1, nb_options):
             opt_desc = rawapi.sane_get_option_descriptor(sane_dev_handle[1], opt_idx)
-            if not rawapi.SaneValueType(opt_desc.type).can_getset_opt():
+            if not SaneValueType(opt_desc.type).can_getset_opt():
                 continue
             opt = ScannerOption.build_from_rawapi(self, opt_idx, opt_desc)
             self.__options[opt.name] = opt
@@ -301,7 +321,9 @@ class Scanner(object):
     options = property(_get_options)
 
     def scan(self, multiple=False):
-        if (not multiple) or self.options['source'].value != "ADF":
+        if (not multiple
+            or (not "ADF" in self.options['source'].value
+                and not "Feeder" in self.options['source'].value)):
             # XXX(Jflesch): We cannot use MultiScanSession() with something
             # else than an ADF. If we try, we will never get
             # SANE_STATUS_NO_DOCS from sane_start()/sane_read() and we will
